//+------------------------------------------------------------------+
//|                                                   TrailingCT.mqh |
//|                   Copyright 2009-2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Expert\ExpertTrailing.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Trailing Stop based on 'Category Theory r2'                |
//| Type=Trailing                                                    |
//| Name=CategoryTheory                                              |
//| ShortName=CT                                                     |
//| Class=CTrailingCT                                                |
//| Page=trailing_ct                                                 |
//| Parameter=Periods,int,14,Period of calculation                   |
//| Parameter=SlopeMA,double,1.0,Slope MA Coeff.                     |
//| Parameter=InterceptMA,int,0,Intercept MA Coeff.                  |
//| Parameter=SlopeATR,double,1.0,Slope ATR Coeff.                   |
//| Parameter=InterceptATR,int,0,Intercept ATR Coeff.                |
//| Parameter=SlopeMA_Period,double,1.0,Slope MA Coeff.                     |
//| Parameter=InterceptMA_Period,int,0,Intercept MA Coeff.                  |
//| Parameter=SlopeATR_Period,double,1.0,Slope ATR Coeff.                   |
//| Parameter=InterceptATR_Period,int,0,Intercept ATR Coeff.                |
//| Parameter=SlopeMA_Price,double,1.0,Slope MA Coeff.                     |
//| Parameter=InterceptMA_Price,int,0,Intercept MA Coeff.                  |
//| Parameter=SlopeATR_Price,double,1.0,Slope ATR Coeff.                   |
//| Parameter=InterceptATR_Price,int,0,Intercept ATR Coeff.                |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CTrailingCT.                                               |
//| Appointment: Class traling stops with 'Category Theory r2'.      |
//| Derives from class CExpertTrailing.                              |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CTrailingCT : public CExpertTrailing
  {
protected:
   //--- adjusted parameters
   int               m_periods;        // the "period of calculation" parameter of the oscillator & indicator
   ENUM_APPLIED_PRICE m_applied;       // the "prices series" parameter of the oscillator & indicator
   
   double            m_slope_ma;             // long ma weight
   int               m_intercept_ma;         // short ma weight
   double            m_slope_atr;            // long atr weight
   int               m_intercept_atr;        // short atr weight
   
   double            m_slope_ma_period;      // slope ma period
   int               m_intercept_ma_period;  // short ma weight
   double            m_slope_atr_period;     // long atr weight
   int               m_intercept_atr_period; // short atr weight
   
   double            m_slope_ma_price;      // slope ma period
   int               m_intercept_ma_price;  // short ma weight
   double            m_slope_atr_price;     // long atr weight
   int               m_intercept_atr_price; // short atr weight

public:
   //--- methods of setting adjustable parameters
   void              Periods(int value)                { m_periods=value;  }
   
   void              SlopeMA(double value)   { m_slope_ma=value;        }
   void              InterceptMA(int value)  { m_intercept_ma=value;    }
   void              SlopeATR(double value)  { m_slope_atr=value;       }
   void              InterceptATR(int value) { m_intercept_atr=value;   }
   
   void              SlopeMA_Period(double value)   { m_slope_ma_period=value;        }
   void              InterceptMA_Period(int value)  { m_intercept_ma_period=value;    }
   void              SlopeATR_Period(double value)  { m_slope_atr_period=value;       }
   void              InterceptATR_Period(int value) { m_intercept_atr_period=value;   }
   
   void              SlopeMA_Price(double value)   { m_slope_ma_price=value;        }
   void              InterceptMA_Price(int value)  { m_intercept_ma_price=value;    }
   void              SlopeATR_Price(double value)  { m_slope_atr_price=value;       }
   void              InterceptATR_Price(int value) { m_intercept_atr_price=value;   }
   
   //--- method of verification of settings
   virtual bool      ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool      InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual bool      CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp);
   virtual bool      CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp);
   //---
                     CTrailingCT(void);
                    ~CTrailingCT(void);

protected:
   
   virtual void      C_Price_Domain(double &ATR_Price,double &MA_Price);
  };
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CTrailingCT::CTrailingCT(void)
  {
//--- initialization of protected data
   m_used_series=USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE+USE_SERIES_TIME;
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CTrailingCT::~CTrailingCT(void)
  {
  }
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CTrailingCT::ValidationSettings(void)
  {
//--- validation settings of additional filters
   if(!CExpertTrailing::ValidationSettings())
      return(false);
//--- 
   if(m_periods<=1)
      return(false);

//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CTrailingCT::InitIndicators(CIndicators *indicators)
  {
//--- check pointer
   if(indicators==NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertTrailing::InitIndicators(indicators))
      return(false);
//--- 
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Checking trailing stop and/or profit for long position.          |
//+------------------------------------------------------------------+
bool CTrailingCT::CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp)
  {
//--- check
      if(position==NULL)
         return(false);
//---
      double _atr_price=0.0,_ma_price=0.0;
      C_Price_Domain(_atr_price,_ma_price);
      
      double _ma=((_ma_price*m_slope_ma_price)+(m_intercept_ma_price*m_symbol.Point()))*((pow(m_periods,-1.0)*m_slope_ma_period)+(m_intercept_ma_period*m_symbol.Point()));
      double _atr=((_atr_price*m_slope_atr_price)+(m_intercept_atr_price*m_symbol.Point()))*((pow(m_periods,-1.0)*m_slope_atr_period)+(m_intercept_atr_period*m_symbol.Point()));
      
      double _sl=((_ma*m_slope_ma)+(m_intercept_ma*m_symbol.Point()))+((_atr*m_slope_atr)+(m_intercept_atr*m_symbol.Point()));
      
      double level =NormalizeDouble(m_symbol.Bid()-m_symbol.StopsLevel()*m_symbol.Point(),m_symbol.Digits());
      double new_sl=NormalizeDouble(_sl,m_symbol.Digits());
      double pos_sl=position.StopLoss();
      double base  =(pos_sl==0.0) ? position.PriceOpen() : pos_sl;
   //---
      sl=EMPTY_VALUE;
      tp=EMPTY_VALUE;
      if(new_sl>base && new_sl<level)
         sl=new_sl;
//---
   return(sl!=EMPTY_VALUE);
  }
//+------------------------------------------------------------------+
//| Checking trailing stop and/or profit for short position.         |
//+------------------------------------------------------------------+
bool CTrailingCT::CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp)
  {
//--- check
      if(position==NULL)
         return(false);
      
      double _atr_price=0.0,_ma_price=0.0;
      C_Price_Domain(_atr_price,_ma_price);
      
      double _ma=((_ma_price*m_slope_ma_price)+(m_intercept_ma_price*m_symbol.Point()))*((pow(m_periods,-1.0)*m_slope_ma_period)+(m_intercept_ma_period*m_symbol.Point()));
      double _atr=((_atr_price*m_slope_atr_price)+(m_intercept_atr_price*m_symbol.Point()))*((pow(m_periods,-1.0)*m_slope_atr_period)+(m_intercept_atr_period*m_symbol.Point()));
      
      double _sl=((_ma*m_slope_ma)+(m_intercept_ma*m_symbol.Point()))+((_atr*m_slope_atr)+(m_intercept_atr*m_symbol.Point()));
      
      double level =NormalizeDouble(m_symbol.Ask()+m_symbol.StopsLevel()*m_symbol.Point(),m_symbol.Digits());
      double new_sl=NormalizeDouble(_sl+m_symbol.Spread()*m_symbol.Point(),m_symbol.Digits());
      double pos_sl=position.StopLoss();
      double base  =(pos_sl==0.0) ? position.PriceOpen() : pos_sl;
//---
      sl=EMPTY_VALUE;
      tp=EMPTY_VALUE;
      if(new_sl<base && new_sl>level)
         sl=new_sl;
//---
      return(sl!=EMPTY_VALUE);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CTrailingCT::C_Price_Domain(double &ATR_Price,double &MA_Price)
   {
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      m_close.Refresh(-1);
      
      double _atr=0.0,_ma=0.0;
      
      int _start=StartIndex()+1;
      
      for(int p=_start;p<m_periods+_start;p++)
      {
         _atr+=(m_high.GetData(p)-m_low.GetData(p));
         
         _ma+=m_close.GetData(p-1);
      }
      
      _atr/=m_periods;
      
      ATR_Price=(m_high.GetData(_start-1)-m_low.GetData(_start-1))+(_atr*(m_periods-1));
      
      MA_Price=_ma;
   }
//+------------------------------------------------------------------+
